/*	Copyright (C) 2004 Bart
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef WIN_GPARTED
#define WIN_GPARTED

#include "../include/Device.h"
#include "../include/VBox_VisualDisk.h"
#include "../include/Partition.h"
#include "../include/TreeView_Detail.h"
#include "../include/Dialog_Partition_Info.h"
#include "../include/Dialog_Partition_New.h"
#include "../include/Operation.h"
#include "../include/Dialog_Progress.h"
#include "../include/Dialog_Partition_Resize_Move.h"
#include "../include/Dialog_About.h"

#include <sigc++/class_slot.h>
#include <gtkmm/main.h>
#include <gtkmm/paned.h>
#include <gtkmm/toolbar.h>
#include <gtkmm/separatortoolitem.h>
#include <gtkmm/menubar.h>
#include <gtkmm/expander.h>
#include <gtkmm/statusbar.h>
#include <gtkmm/radiobutton.h>
#include <gtkmm/liststore.h>
#include <gtkmm/scrolledwindow.h>

#include <fstream>

namespace GParted
{
	
class Win_GParted : public Gtk::Window
{
public:
	Win_GParted(   );

private:
	void Find_Devices() ;
	
	//Fill txtview_device_info_buffer with some information about the selected device
	void Fill_Label_Device_Info( );

	//overridden signalhandler
	virtual bool on_delete_event(GdkEventAny* ) ;

	void Add_Operation( OperationType, const Partition & );
	void Refresh_Visual( );
	bool Quit_Check_Operations();
	void Set_Valid_Operations() ; //determines which operations are allowed on selected_partition 
	void Fill_Convert_Menu() ;

	//convenience functions
	void allow_resize( bool b ) 	{ menu_popup.items()[0].set_sensitive( b ); toolbar_main.get_nth_item(0) ->set_sensitive( b ); }
	void allow_copy( bool b )		{ menu_popup.items()[1].set_sensitive( b ); toolbar_main.get_nth_item(1) ->set_sensitive( b ); }
	void allow_new( bool b )		{ menu_popup.items()[3].set_sensitive( b ); toolbar_main.get_nth_item(3) ->set_sensitive( b ); }
	void allow_delete( bool b ) 	{ menu_popup.items()[4].set_sensitive( b ); toolbar_main.get_nth_item(4) ->set_sensitive( b ); }
	void allow_info( bool b ) 		{ menu_popup.items()[6].set_sensitive( b ); }//toolbar_main.get_nth_item(6) ->set_sensitive( b ); }
	void allow_convert( bool b )	{ menu_popup.items()[8].set_sensitive( b ); }
	
	//signal handlers
	void optionmenu_devices_changed( );
	
	void menu_gparted_reread_devices();
	void menu_gparted_quit();
	void menu_help_contents();
	void menu_help_about();

	void mouse_click( GdkEventButton*,  const Partition & );
		
	void activate_resize(); 
	void activate_copy();
	void activate_new();
	void activate_delete();
	void activate_info();
	void activate_convert( const Glib::ustring & new_fs  );
	
	void activate_undo();
	void activate_apply();
	void apply_operations_thread();

//private variables
	int current_device;
	Partition selected_partition;
	std::vector <Device * > devices;
	std::vector <Operation> operations;

	Gtk::HPaned hpaned_main;
	Gtk::VBox vbox_main,vbox_info;
	Gtk::HBox hbox_toolbar,hbox_visual;
	Gtk::Toolbar toolbar_main;
	Gtk::MenuBar menubar_main;
	Gtk::OptionMenu optionmenu_devices;
	Gtk::Menu menu_devices,menu_gparted,menu_help,menu_popup, menu_convert;
	Gtk::ToolButton *toolbutton;
	Gtk::Statusbar statusbar;
	Gtk::Expander expander_device_info,expander_pending_operations;
	Gtk::Image *image_temp;
	Gtk::ScrolledWindow scrollwindow_left,scrollwindow_right;
	Gtk::Label label_device_info;
	Gtk::Tooltips tooltips ;
			
	VBox_VisualDisk *vbox_visual_disk;
	TreeView_Detail *treeview_detail;
	
	//operations list
	Gtk::TreeView treeview_operations;
	Glib::RefPtr<Gtk::ListStore> liststore_operations;
	
	struct treeview_operations_Columns : public Gtk::TreeModelColumnRecord             
	{
		Gtk::TreeModelColumn<int> operation_number;
		Gtk::TreeModelColumn<Glib::ustring> operation_description;
		Gtk::TreeModelColumn< Glib::RefPtr<Gdk::Pixbuf> >  operation_icon;
				
		treeview_operations_Columns() {	add( operation_number ); add( operation_description );add(operation_icon);}
	};
	treeview_operations_Columns treeview_operations_columns;
	
	
	//usefull variables which are used by many different functions...
	
	sigc::connection s1,s2,s3;//usefull for disconnecting and destroying a connection ( see optionmenu_devices_changed() and activate_apply() )
	std::ostringstream os;//for int to string conversions
	bool any_logic,any_extended;//used in some checks (e.g. see optionmenu_devices_changed()
	unsigned short highest_logic_busy,primary_count, new_count;//primary_count checks for max. of 4 pimary partitions, 
																								//new_count keeps track of the new created partitions,
																								//highest_logic_busy contains the number of the highest logical which is busy ( 0 if none is busy)
	
	GParted::Device *temp_device;
	std::vector <Glib::ustring> str_devices;
	
	//used to create the convert and/or devices menu items..
	Gtk::MenuItem *item;
	Gtk::Label *label_temp;
	Gtk::Entry *entry_temp;
	Gtk::HBox *hbox_temp;
	Gdk::Color color_temp;
	std::vector <Glib::ustring> filesystems;
	
		
	//stuff for progress overview
	Dialog_Progress *dialog_progress;
	Glib::Thread *thread_operations;
	Glib::Dispatcher dispatcher_next_operation;
	bool apply;
	
};

} //GParted

#endif //WIN_GPARTED
