/* Copyright (C) 2004 Bart
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include "../include/Dialog_Partition_Info.h"

namespace GParted
{

Dialog_Partition_Info::Dialog_Partition_Info( const Partition & partition )
{
	this -> partition = partition ;

	this->set_resizable( false );
	this->set_title( "Information about " + partition.partition );
	 
	drawingarea .set_size_request( 350, 50 ) ;
	drawingarea.signal_realize().connect( sigc::mem_fun(*this, &Dialog_Partition_Info::drawingarea_on_realize) ) ;
	drawingarea.signal_expose_event().connect( sigc::mem_fun(*this, &Dialog_Partition_Info::drawingarea_on_expose) ) ;
	frame_partition .add ( drawingarea ) ;
	
	frame_partition .set_shadow_type(Gtk::SHADOW_ETCHED_OUT);
	frame_partition .set_border_width( 10 );
	hbox_frame .pack_start( frame_partition, Gtk::PACK_EXPAND_PADDING ) ;
	
	this ->get_vbox() ->pack_start( hbox_frame, Gtk::PACK_SHRINK ) ;
	
	//calculate proportional width of used and unused
	used = unused = 0 ;
	used 		= (int ) ( (  (350 - BORDER *2) / ( (double)  (partition.sector_end - partition.sector_start) / partition.sectors_used ) )+0.5 ) ;
	unused  	= 350 - used - (BORDER *2) ;
	
	//allocate some colors
	color_used.set( "#F8F8BA" );						this ->get_colormap() ->alloc_color( color_used ) ;
	
	partition .type == GParted::EXTENDED ?  color_unused.set( "darkgrey" ) : color_unused.set( "white" ) ;
	this ->get_colormap() ->alloc_color( color_unused ) ;

	color_text.set( "black" );								this ->get_colormap() ->alloc_color( color_text ) ;
	color_partition = partition.color ;							this ->get_colormap() ->alloc_color( color_partition ) ;	 
	
	//set text of pangolayout
	os << partition .partition << "\n" << this -> partition .Get_Length_MB() << " MB";
	pango_layout = drawingarea .create_pango_layout ( os.str() ) ;os.str("");
	
	//add label for detail and fill with relevant info
	label_info .set_alignment( Gtk::ALIGN_LEFT   );
	Display_Info() ;
	this ->get_vbox() ->pack_start( label_info, Gtk::PACK_SHRINK ) ;
	
	//display error (if any)
	if ( partition .error != "" )
	{
		frame_error = manage( new Gtk::Frame() );
		frame_error ->set_border_width( 10 );
		
		image_temp = manage( new Gtk::Image( Gtk::Stock::DIALOG_WARNING, Gtk::ICON_SIZE_BUTTON ) );
		label_temp = manage( new Gtk::Label( ) ) ;
		label_temp ->set_markup( "<b> Libparted message: </b>" ) ;
		
		hbox_temp = manage( new Gtk::HBox() );
		hbox_temp ->pack_start( *image_temp, Gtk::PACK_SHRINK ) ;
		hbox_temp ->pack_start( *label_temp, Gtk::PACK_SHRINK ) ;
		
		frame_error ->set_label_widget( *hbox_temp ) ;
		
		label_temp = manage( new Gtk::Label( ) ) ;
		label_temp ->set_markup( "<i>" +  partition.error + "</i>") ;
		label_temp ->set_line_wrap( true ) ;
		frame_error ->add( *label_temp );
		
		this ->get_vbox() ->pack_start( *frame_error, Gtk::PACK_SHRINK ) ;
		
		
	}
		
		
	this->add_button( Gtk::Stock::CLOSE, Gtk::RESPONSE_OK );
	this->show_all_children();
}

void Dialog_Partition_Info::drawingarea_on_realize(  )
{
	gc = Gdk::GC::create( drawingarea .get_window() );
	
	drawingarea .get_window() ->set_background( color_partition );
	
}

bool Dialog_Partition_Info::drawingarea_on_expose( GdkEventExpose *ev  )
{
	if ( partition.type != GParted::UNALLOCATED ) 
	{
		//used
		gc ->set_foreground(  color_used );
		drawingarea .get_window() ->draw_rectangle( gc, true, BORDER, BORDER, used ,34 );
		
		//unused
		gc ->set_foreground(  color_unused );
		drawingarea .get_window() ->draw_rectangle( gc, true, BORDER + used, BORDER, unused,34 );
	}
	
	//text
	gc ->set_foreground(  color_text );
	drawingarea .get_window() ->draw_layout( gc, BORDER +5, BORDER +1 , pango_layout ) ;
	
	return true;
}

void Dialog_Partition_Info::Display_Info()
{
	os << "   <b>Filesystem:</b>\t" << partition.filesystem << "  \n";
	os << "   <b>Size:</b>\t\t\t" << this -> partition .Get_Length_MB() << " MB   \n";
	
	if ( partition.sectors_used != -1 )
	{ 
		int percent_used =(int)  (  (double) partition.Get_Used_MB()  / partition .Get_Length_MB() *100 +0.5 ) ;
		os << "   <b>Used:</b>\t\t\t" <<  this ->partition .Get_Used_MB() << " MB\t( " << percent_used << "% )\n";
		os << "   <b>Unused:</b>\t\t" <<  this ->partition .Get_Unused_MB() << " MB\t( " << 100 - percent_used << "% )\n";
	}
	
	if ( partition .flags != "" )
		os << "   <b>Flags:</b>\t\t\t" << partition .flags << "\n";
	
	os << "\n";
	
	if ( partition.type != GParted::UNALLOCATED && partition .partition.substr( 0, 3 ) != "New" )
	{
		os << "   <b>Path:</b>\t\t\t" << partition.partition << "   \n";
		
		//get realpath
		char real_path[4096] ;
		realpath( partition.partition.c_str() , real_path );
		
		//only show realpath if it's diffent from the short path...
		if ( partition.partition != real_path )
			os << "   <b>Real Path:</b>\t\t" << (Glib::ustring) real_path << "   \n";
	}
	if ( partition.type != GParted::UNALLOCATED && partition .partition.substr( 0, 3 ) != "New" )
	{
		os << "   <b>Status:</b>\t\t" ;
		if ( partition.busy )
			Find_Status() ;
		else if ( partition.type == GParted::EXTENDED )
			os << "Not busy (There are no mounted logical partitions)   \n";
		else if ( partition.filesystem == "linux-swap" )
			os << "Not active   \n";
		else 
			os << "Not mounted   \n";
	}
	os << "\n";
	
	//and some sector information
	os << "   <b>First Sector:</b>\t" << partition.sector_start << "   \n";
	os << "   <b>Last Sector:</b>\t" << partition.sector_end << "   \n";
	os << "   <b>Total Sectors:</b>\t" << partition.sector_end - partition.sector_start << "   \n";
	
	label_info .set_markup( os.str() ) ; os.str( "" ) ;
}

void Dialog_Partition_Info::Find_Status() 
{
	if ( partition.type == GParted::EXTENDED )
	{
		os <<  "Busy  (At least one logical partition is mounted)   \n";
		return ;
	}
	
	
	if ( partition.filesystem == "linux-swap" )
	{
		os << "Active    \n";
		return ;
	}
	
	//try to find the mountpoint in /proc/mounts
	//get realpath
	char real_path[4096] ;
	realpath( partition.partition.c_str() , real_path );
	Glib::ustring mountpoint, partition_real_path = real_path ; //because root partition is listed as /dev/root we need te compare against te real path..
	
	
	std::ifstream file_mounts( "/proc/mounts" ) ;
	std::string line ;
	
	while ( getline( file_mounts, line ) )
	{
		realpath( line.substr( 0, line.find( ' ' ) ) .c_str() , real_path );
		
		if ( partition_real_path == real_path )
		{
			mountpoint = line.substr( line.find( ' ' ) +1, line .length()  ) ;
			os << "Mounted on "  <<  mountpoint .substr( 0, mountpoint .find( ' ' ) )  << "   \n";
			break ;
		}
		
		
	}
	
	file_mounts .close() ;

	//sometimes rootdevices are not listed as paths. I'll take a guess and just enter / here...( we'll look into this when complaints start coming in :P )
	if ( mountpoint.empty() )
		os << "Mounted on /   \n";

}

Dialog_Partition_Info::~Dialog_Partition_Info()
{
	this ->get_colormap() ->free_colors( color_used, 1 ) ;
	this ->get_colormap() ->free_colors( color_unused, 1 ) ;
	this ->get_colormap() ->free_colors( color_text, 1 ) ;
	this ->get_colormap() ->free_colors( color_partition, 1 ) ;
}


} //GParted
